/*#############################################################
Driver name	    : RGBMatrix64x32_F4.c
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the 64x32 RGB LED
									Matrix by Adafruit (https://www.adafruit.com/
									products/2278).

Requirements    : * STM32F4-Discovery Board
									* Define the SET and RESET functions for the
										output port pins.
									* Do the proper GPIO initialization for the
										output port pins in RGBRGBMatrix64x32_Init();

Functions				: RGBMatrix64x32_Init
									RGBMatrix64x32_ClrScr
									RGBMatrix64x32_DrawPixel
									RGBMatrix64x32_SetContrast
													  
Special Note(s) : NONE
##############################################################*/



/* //////////////////////////////////////////////////////////////
Define the SET and RESET functions for each of the output port 
pins for the RGBMatrix64x32 here.  
Also do the proper GPIO initialization for these port pins in
RGBMatrix64x32_Init().
////////////////////////////////////////////////////////////// */
#define RGBMatrix64x32_R1_1			GPIO_WriteBit(GPIOE, GPIO_Pin_15, Bit_SET)		//R1 = 1
#define RGBMatrix64x32_R1_0			GPIO_WriteBit(GPIOE, GPIO_Pin_15, Bit_RESET)	//R1 = 0
#define RGBMatrix64x32_G1_1			GPIO_WriteBit(GPIOE, GPIO_Pin_14, Bit_SET)		//G1 = 1
#define RGBMatrix64x32_G1_0			GPIO_WriteBit(GPIOE, GPIO_Pin_14, Bit_RESET)	//G1 = 0
#define RGBMatrix64x32_B1_1			GPIO_WriteBit(GPIOE, GPIO_Pin_13, Bit_SET)		//B1 = 1
#define RGBMatrix64x32_B1_0			GPIO_WriteBit(GPIOE, GPIO_Pin_13, Bit_RESET)	//B1 = 0
#define RGBMatrix64x32_R2_1			GPIO_WriteBit(GPIOE, GPIO_Pin_12, Bit_SET)		//R2 = 1
#define RGBMatrix64x32_R2_0			GPIO_WriteBit(GPIOE, GPIO_Pin_12, Bit_RESET)	//R2 = 0
#define RGBMatrix64x32_G2_1			GPIO_WriteBit(GPIOE, GPIO_Pin_11, Bit_SET)		//G2 = 1
#define RGBMatrix64x32_G2_0			GPIO_WriteBit(GPIOE, GPIO_Pin_11, Bit_RESET)	//G2 = 0
#define RGBMatrix64x32_B2_1			GPIO_WriteBit(GPIOE, GPIO_Pin_10, Bit_SET)		//B2 = 1
#define RGBMatrix64x32_B2_0			GPIO_WriteBit(GPIOE, GPIO_Pin_10, Bit_RESET)	//B2 = 0
#define RGBMatrix64x32_CLK_1		GPIO_WriteBit(GPIOE, GPIO_Pin_9, Bit_SET)			//CLK = 1
#define RGBMatrix64x32_CLK_0		GPIO_WriteBit(GPIOE, GPIO_Pin_9, Bit_RESET)		//CLK = 0
#define RGBMatrix64x32_A_1			GPIO_WriteBit(GPIOD, GPIO_Pin_11, Bit_SET)		//A = 1
#define RGBMatrix64x32_A_0			GPIO_WriteBit(GPIOD, GPIO_Pin_11, Bit_RESET)	//A = 0
#define RGBMatrix64x32_B_1			GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_SET)		//B = 1
#define RGBMatrix64x32_B_0			GPIO_WriteBit(GPIOD, GPIO_Pin_10, Bit_RESET)	//B = 0
#define RGBMatrix64x32_C_1			GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_SET)			//C = 1
#define RGBMatrix64x32_C_0			GPIO_WriteBit(GPIOD, GPIO_Pin_9, Bit_RESET)		//C = 0
#define RGBMatrix64x32_D_1			GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_SET)			//D = 1
#define RGBMatrix64x32_D_0			GPIO_WriteBit(GPIOD, GPIO_Pin_8, Bit_RESET)		//D = 0
#define RGBMatrix64x32_TP_1			GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_SET)			//TP = 1
#define RGBMatrix64x32_TP_0			GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_RESET)		//TP = 0
#define RGBMatrix64x32_OE_1			GPIO_WriteBit(GPIOD, GPIO_Pin_6, Bit_SET)			//OE = 1
#define RGBMatrix64x32_OE_0			GPIO_WriteBit(GPIOD, GPIO_Pin_6, Bit_RESET)		//OE = 0



/* #global variables -----------------------------------------*/
uint8_t RGBMatrix64x32_RAM[32][64];
uint8_t RGBMatrix64x32_ROW=0;
uint16_t RGBMatrix64x32_CONTRAST=0;				//0 = max; 4200 = min



void RGBMatrix64x32_Init(void);
void RGBMatrix64x32_ClrScr(uint8_t color);
void RGBMatrix64x32_DrawPixel(uint8_t x, uint8_t y, uint8_t color);



/*********************************************************************************************
Function name   : RGBMatrix64x32_Init
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the RGBMatrix64x32 module

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void RGBMatrix64x32_Init(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;		
	NVIC_InitTypeDef NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_OCInitTypeDef				TIM_OCInitStructure;							//structure for TIM Output Compare
	
	/* GPIOD and GPIOE Periph clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD | RCC_AHB1Periph_GPIOE, ENABLE);		//enable the AHB1 bus to use GPIOD and GPIOE
  
  /* Configure PD11 - PD6 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11| GPIO_Pin_10 | 
																GPIO_Pin_9 | GPIO_Pin_8 | 
																GPIO_Pin_7 | GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_25MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOD, &GPIO_InitStructure);		

  /* Configure PE15 - PE9 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_14 | 
																GPIO_Pin_13 | GPIO_Pin_12 | 
																GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_25MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOE, &GPIO_InitStructure);	
	
	/* TIM13 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM13, ENABLE);

  /* Enable the TIM13 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM8_UP_TIM13_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 83;  										//PSC value; meanse that TIM register will be incremented every 1us (84000000 / (83+1) = 1000000Hz = 1us
  TIM_TimeBaseStructure.TIM_Period = (16000 / 16) - 1;					//delay until next line is drawn is (refresh rate / row numbers) - 1
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM13, &TIM_TimeBaseStructure);

	/* TIM Interrupts enable */
  TIM_ITConfig(TIM13, TIM_IT_Update, ENABLE);

  /* TIM enable counter */
  TIM_Cmd(TIM13, ENABLE);
	


	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);			//enable the AHB1 Peripheral Clock to use GPIOB
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);			//enable the TIM3 clock
	
	/* Pin configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;									//configure PB5 in alternate function pushpull mode
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;							
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;							
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;				
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;							
  GPIO_Init(GPIOB, &GPIO_InitStructure);										//initialize GPIOB with the above structure
	
	/* Connect TIM3 pins to AF */ 
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_TIM3);		//TIM3 Channel 2
	
	//Time base configuration 
	//The MCU on the STM32F4-Discovery board is clocked at 168Mhz and we need a 50Hz timebase (20000us period)
	//TIM_CLK = 168Mhz / 2 = 84Mhz
	//Update_event = TIM_CLK /((PSC + 1)*(ARR   + 1)*(RCR + 1))
	//			 20000 = 84000000/((0   + 1)*(4199  + 1)*(0   + 1))
	//We need to use the maximum value for ARR (<= 65536) to get the max resolution to control the PWM output.  (0 - 0%, 2100 = 50%, 4200 = 100% in this case)
	
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 4199;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  /* PWM1 Mode configuration: Channel2 */
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;				//0% duty cycle 
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  TIM_OC2Init(TIM3, &TIM_OCInitStructure);
  TIM_OC2PreloadConfig(TIM3, TIM_OCPreload_Enable);

  TIM_ARRPreloadConfig(TIM3, ENABLE);

  /* TIM3 enable counter */
  TIM_Cmd(TIM3, ENABLE);
}



/*********************************************************************************************
Function name   : RGBMatrix64x32_ClrScr
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Fills the display with a certain colour.

Special Note(s) : NONE

Parameters			: color		-	8-bit value; 0=black,1=blue,2=green,3=cyan,4=red,5=magenta,
														6=yellow,7=white
Return value		: NONE
*********************************************************************************************/
void RGBMatrix64x32_ClrScr(uint8_t color)
{
	uint8_t x,y;
	
	for(y=0; y<32; y++)
		for(x=0; x<64; x++)
			RGBMatrix64x32_RAM[y][x] = color;
}



/*********************************************************************************************
Function name   : RGBMatrix64x32_DrawPixel
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a pixel at the specified xy position

Special Note(s) : NONE

Parameters			: x				-	x position (0 - 64)
									y				-	y position (0 - 32)
									color		-	color of the pixel; 0=black,1=blue,2=green,3=cyan,4=red,5=magenta,
													  6=yellow,7=white
Return value		: NONE
*********************************************************************************************/
void RGBMatrix64x32_DrawPixel(uint8_t x, uint8_t y, uint8_t color)
{
	if((x<64) && (y<32))
	{
		RGBMatrix64x32_RAM[y][x] = color;
	}
}



/*********************************************************************************************
Function name   : RGBMatrix64x32_SetContrast
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the contrast of the RGBMatrix64x32

Special Note(s) : NONE

Parameters			: value		-	8-bit value representing the brightness (0 - 100)
Return value		: NONE
*********************************************************************************************/
void RGBMatrix64x32_SetContrast(uint8_t value) 
{
	if(value <= 100)
	{
		value = 100 - value;
		RGBMatrix64x32_CONTRAST = 42 * value;
	}
}



/*********************************************************************************************
Function name   : TIM8_UP_TIM13_IRQHandler
Author 					: Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Handles the interrupt for the TIM13 update event.  This handler will update
									a single line on the RGBMatrix64x32 module.

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void TIM8_UP_TIM13_IRQHandler(void)
{
	int x;
	uint16_t j;
	
	TIM_ClearITPendingBit(TIM13, TIM_IT_Update);				//clear interrupt flag
	
	TIM_SetCompare2(TIM3, 4200);	//RGBMatrix64x32_OE_1;						//disable LEDs
	
	/* fill the current row with the corresponding LEDS */
	for(x=0; x<64; x++)
	{
		//Set/Clear R1,R2, G1,G2, B1,B2
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW][x] & 0x01)
				RGBMatrix64x32_B1_1;
		else
				RGBMatrix64x32_B1_0;
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW][x] & 0x02)
				RGBMatrix64x32_G1_1;
		else
				RGBMatrix64x32_G1_0;
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW][x] & 0x04)
				RGBMatrix64x32_R1_1;
		else
				RGBMatrix64x32_R1_0;
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW+16][x] & 0x01)
				RGBMatrix64x32_B2_1;
		else
				RGBMatrix64x32_B2_0;
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW+16][x] & 0x02)
				RGBMatrix64x32_G2_1;
		else
				RGBMatrix64x32_G2_0;
		if(RGBMatrix64x32_RAM[RGBMatrix64x32_ROW+16][x] & 0x04)
				RGBMatrix64x32_R2_1;
		else
				RGBMatrix64x32_R2_0;
		
		//clock the new data into the shift register
		for(j=0;j<100;j++);
		RGBMatrix64x32_CLK_1;
		for(j=0;j<100;j++);
		RGBMatrix64x32_CLK_0;
	}

	//Make all row control lines 1's first (dunno why)
//	RGBMatrix64x32_A_1;
//	RGBMatrix64x32_B_1;
//	RGBMatrix64x32_C_1;
//	RGBMatrix64x32_D_1;
	
	//Then set/clear row control lines
	if(RGBMatrix64x32_ROW == 0)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 1)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 2)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 3)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 4)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 5)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 6)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 7)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_0;
	}
	else if(RGBMatrix64x32_ROW == 8)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 9)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 10)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 11)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_0;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 12)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 13)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_0;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 14)
	{
		RGBMatrix64x32_A_0;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_1;
	}
	else if(RGBMatrix64x32_ROW == 15)
	{
		RGBMatrix64x32_A_1;
		RGBMatrix64x32_B_1;
		RGBMatrix64x32_C_1;
		RGBMatrix64x32_D_1;
	}
	
	//RGBMatrix64x32_OE_1;	//TIM_SetCompare2(TIM3, 8000);
		
	RGBMatrix64x32_TP_1;							//latch the new data
	RGBMatrix64x32_TP_0;
	TIM_SetCompare2(TIM3, RGBMatrix64x32_CONTRAST);	//RGBMatrix64x32_OE_0; 												//enable the LEDs				

	//increment scan row
	RGBMatrix64x32_ROW++;
	if(RGBMatrix64x32_ROW > (16 - 1))
		RGBMatrix64x32_ROW = 0;
}
