/*#############################################################
File Name       : ex1_RGBMatrix64x32_colorcontrast.c
Author          : Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Demonstrates the use of 64x32 RGB LED Matrix 
									by Adafruit (https://www.adafruit.com/
									products/2278).  It shows how to use the 
									RGBMatrix64x32_DrawPixel function to draw a
									dot in a certain on the display, and also the 
									RGBMatrix64x32_SetContrast function to set the
									contrast of the display.

Requirements    : * STM32F4-Discovery Board
									* Make sure the RGBMatrix64x32_F4.c file is in the 
                    same folder than the project
              
Circuit         : * A LED matrix display with connected as follows:
										
										R1	-	PE15
										G1	-	PE14
										B1	-	PE13
										R2	-	PE12
										G2	-	PE11
										B2	-	PE10
										CLK	-	PE9
										
										A		-	PD11
										B		-	PD10
										C		-	PD9
										D		-	PD8
										TP	-	PD7
										
										OE	-	RB5	(PWM using Timer 3 for contrast)
											

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision
#include "RGBMatrix64x32_F4.c"																	//for the RGBMatrix64x32 functions using the STM32F4-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	int x, y, contrast;
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	RGBMatrix64x32_Init();				//initialize the RGBMatrix64x32 module

	RGBMatrix64x32_ClrScr(0);			//clear the display with BLACK pixels

	/* draw a BLUE block from (0,0) - (8,15) */
	for(x=0;x<=8;x++)
		for(y=0; y<=15; y++)
			RGBMatrix64x32_DrawPixel(x,y,1);
	
	/* draw a GREEN block from (9,16) - (17,31) */
	for(x=9;x<=17;x++)
		for(y=16; y<=31; y++)
			RGBMatrix64x32_DrawPixel(x,y,2);
	
	/* draw a CYAN block from (18,0) - (26,15) */
	for(x=18;x<=26;x++)
		for(y=0; y<=15; y++)
			RGBMatrix64x32_DrawPixel(x,y,3);
	
	/* draw a RED block from (27,16) - (35,31) */
	for(x=27;x<=35;x++)
		for(y=16; y<=31; y++)
			RGBMatrix64x32_DrawPixel(x,y,4);
			
	/* draw a MAGENTA block from (36,0) - (44,15) */
	for(x=36;x<=44;x++)
		for(y=0; y<=15; y++)
			RGBMatrix64x32_DrawPixel(x,y,5);
			
	/* draw a YELLOW block from (45,16) - (53,31) */
	for(x=45;x<=53;x++)
		for(y=16; y<=31; y++)
			RGBMatrix64x32_DrawPixel(x,y,6);

	/* draw a WHITE block from (54,0) - (62,15) */
	for(x=54;x<=62;x++)
		for(y=0; y<=15; y++)
			RGBMatrix64x32_DrawPixel(x,y,7);
			
			
	/* Main program loop */
  while (1)
  {
		/* decrement the contrast of the display by 1 every 100ms */
		for(contrast=100; contrast >=0; contrast=contrast-1)
		{
			RGBMatrix64x32_SetContrast(contrast);		//set the contrast of the display
			Delay(100);
		}
		
		/* increment the contrast of the display by 1 every 100ms */
		for(contrast=0; contrast <=100; contrast=contrast+1)
		{
			RGBMatrix64x32_SetContrast(contrast);		//set the contrast of the display
			Delay(100);
		}
  }		
}













