/*#############################################################
File Name       : ex2_RGBMatrix64x32_ABC.c
Author          : Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Demonstrates the use of 64x32 RGB LED Matrix 
									by Adafruit (https://www.adafruit.com/
									products/2278).  It shows how to use the 
									RGBMatrix64x32_DrawPixel function to draw 
									characters and lines.

Requirements    : * STM32F4-Discovery Board
									* Make sure the RGBMatrix64x32_F4.c file is in the 
                    same folder than the project
              
Circuit         : * A LED matrix display with connected as follows:
										
										R1	-	PE15
										G1	-	PE14
										B1	-	PE13
										R2	-	PE12
										G2	-	PE11
										B2	-	PE10
										CLK	-	PE9
										
										A		-	PD11
										B		-	PD10
										C		-	PD9
										D		-	PD8
										TP	-	PD7
										
										OE	-	RB5	(PWM using Timer 3 for contrast)
											

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision
#include "RGBMatrix64x32_F4.c"																	//for the RGBMatrix64x32 functions using the STM32F4-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void DrawA(uint8_t x, uint8_t y, uint8_t color);
void DrawB(uint8_t x, uint8_t y, uint8_t color);
void DrawC(uint8_t x, uint8_t y, uint8_t color);

int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	int x,y;
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	RGBMatrix64x32_Init();				//initialize the RGBMatrix64x32 module

	RGBMatrix64x32_ClrScr(0);			//clear the display with BLACK pixels

	RGBMatrix64x32_SetContrast(100);
			
	/* Draw top and bottom line of box */
	for(x=20;x<44;x++)
	{
		RGBMatrix64x32_DrawPixel(x,9,7);
		RGBMatrix64x32_DrawPixel(x,21,7);
	}
	
	/* Draw left and right line of box */
	for(y=9; y<=21; y++)
	{
		RGBMatrix64x32_DrawPixel(20,y,7);
		RGBMatrix64x32_DrawPixel(43,y,7);
	}
	
	/* Draw ABC in box */
	DrawA(23,12,4);
	DrawB(30,12,2);
	DrawC(37,12,1);
	
	/* Wait 2 sec before start */
	Delay(2000);
	
	/* Main program loop */
  while (1)
  {
		/* Draw ABC in black to remove it */
		DrawA(23,12,0);
		DrawB(30,12,0);
		DrawC(37,12,0);
		
		Delay(500);
		
		/* Draw ABC in color again */
		DrawA(23,12,4);
		DrawB(30,12,2);
		DrawC(37,12,1);
		
		Delay(500);
  }		
}

void DrawA(uint8_t x, uint8_t y, uint8_t color)
{
	RGBMatrix64x32_DrawPixel(x, y+2, color);
	RGBMatrix64x32_DrawPixel(x, y+3, color);
	RGBMatrix64x32_DrawPixel(x, y+4, color);
	RGBMatrix64x32_DrawPixel(x, y+5, color);
	RGBMatrix64x32_DrawPixel(x, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+1, y+1, color);
	RGBMatrix64x32_DrawPixel(x+1, y+4, color);
	
	RGBMatrix64x32_DrawPixel(x+2, y, color);
	RGBMatrix64x32_DrawPixel(x+2, y+4, color);
	
	RGBMatrix64x32_DrawPixel(x+3, y+1, color);
	RGBMatrix64x32_DrawPixel(x+3, y+4, color);
	
	RGBMatrix64x32_DrawPixel(x+4, y+2, color);
	RGBMatrix64x32_DrawPixel(x+4, y+3, color);
	RGBMatrix64x32_DrawPixel(x+4, y+4, color);
	RGBMatrix64x32_DrawPixel(x+4, y+5, color);
	RGBMatrix64x32_DrawPixel(x+4, y+6, color);
}

void DrawB(uint8_t x, uint8_t y, uint8_t color)
{
	RGBMatrix64x32_DrawPixel(x, y, color);
	RGBMatrix64x32_DrawPixel(x, y+1, color);
	RGBMatrix64x32_DrawPixel(x, y+2, color);
	RGBMatrix64x32_DrawPixel(x, y+3, color);
	RGBMatrix64x32_DrawPixel(x, y+4, color);
	RGBMatrix64x32_DrawPixel(x, y+5, color);
	RGBMatrix64x32_DrawPixel(x, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+1, y, color);
	RGBMatrix64x32_DrawPixel(x+1, y+3, color);
	RGBMatrix64x32_DrawPixel(x+1, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+2, y, color);
	RGBMatrix64x32_DrawPixel(x+2, y+3, color);
	RGBMatrix64x32_DrawPixel(x+2, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+3, y, color);
	RGBMatrix64x32_DrawPixel(x+3, y+3, color);
	RGBMatrix64x32_DrawPixel(x+3, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+4, y+1, color);
	RGBMatrix64x32_DrawPixel(x+4, y+2, color);
	RGBMatrix64x32_DrawPixel(x+4, y+4, color);
	RGBMatrix64x32_DrawPixel(x+4, y+5, color);
}

void DrawC(uint8_t x, uint8_t y, uint8_t color)
{
	RGBMatrix64x32_DrawPixel(x, y+1, color);
	RGBMatrix64x32_DrawPixel(x, y+2, color);
	RGBMatrix64x32_DrawPixel(x, y+3, color);
	RGBMatrix64x32_DrawPixel(x, y+4, color);
	RGBMatrix64x32_DrawPixel(x, y+5, color);
	
	RGBMatrix64x32_DrawPixel(x+1, y, color);
	RGBMatrix64x32_DrawPixel(x+1, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+2, y, color);
	RGBMatrix64x32_DrawPixel(x+2, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+3, y, color);
	RGBMatrix64x32_DrawPixel(x+3, y+6, color);
	
	RGBMatrix64x32_DrawPixel(x+4, y+1, color);
	RGBMatrix64x32_DrawPixel(x+4, y+5, color);
}













