/*#############################################################
File Name       : ex3_RGBMatrix64x32_ball.c
Author          : Grant Phillips
Date Modified   : 06/10/2015
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Demonstrates the use of 64x32 RGB LED Matrix 
									by Adafruit (https://www.adafruit.com/
									products/2278) to animate a "bouncing ball".

Requirements    : * STM32F4-Discovery Board
									* Make sure the RGBMatrix64x32_F4.c file is in the 
                    same folder than the project
              
Circuit         : * A LED matrix display with connected as follows:
										
										R1	-	PE15
										G1	-	PE14
										B1	-	PE13
										R2	-	PE12
										G2	-	PE11
										B2	-	PE10
										CLK	-	PE9
										
										A		-	PD11
										B		-	PD10
										C		-	PD9
										D		-	PD8
										TP	-	PD7
										
										OE	-	RB5	(PWM using Timer 3 for contrast)
											

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision
#include "RGBMatrix64x32_F4.c"																	//for the RGBMatrix64x32 functions using the STM32F4-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	int x=0, y=0, xdir=1, ydir=1, c=1;
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	RGBMatrix64x32_Init();				//initialize the RGBMatrix64x32 module

	RGBMatrix64x32_ClrScr(0);			//clear the display with BLACK pixels

	RGBMatrix64x32_SetContrast(10);
			
	/* Main program loop */
  while (1)
  {
		//ball ON at current pos
		RGBMatrix64x32_DrawPixel(x,y, c);	
		RGBMatrix64x32_DrawPixel(x+1,y, c);
		RGBMatrix64x32_DrawPixel(x,y+1, c);
		RGBMatrix64x32_DrawPixel(x+1,y+1, c);		
		
		//wait a bit
		Delay(50);									
		
		//ball OFF at current pos
		RGBMatrix64x32_DrawPixel(x,y,0);	
		RGBMatrix64x32_DrawPixel(x+1,y,0);
		RGBMatrix64x32_DrawPixel(x,y+1,0);
		RGBMatrix64x32_DrawPixel(x+1,y+1,0);
		
		x = x + xdir;								//calculate new x
		if(x >= 62)
		{			
			xdir = -1;
			c++;
		}
		if(x <= 0)
		{
			xdir = 1;
			c++;
		}
		
		y = y + ydir;								//calculate new y
		if(y >= 30)
		{
			ydir = -1;
			c++;
		}
		if(y <= 0)
		{
			ydir = 1;
			c++;
		}
		if(c > 7)
			c = 1;
  }		
}













